import argparse
import sys
from typing import Optional, Sequence
from urllib.request import HTTPError, URLError

from dynmen import MenuError

from .config import write_default
from .settings import CATEGORIES_STRINGS, CONFIG_DIR, CONFIG_FILE
from .tpb import TPB


def parse_args(args: Optional[Sequence[str]] = None) -> argparse.Namespace:
    """Argument parsing.

    Args:
        args: arguments.

    Returns:
        parsed arguments.
    """
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(description="Query ThePirateBay.")
    subparsers = parser.add_subparsers()
    parser.add_argument("-u", "--url", type=str, default=None, help="ThePirateBay url.")

    search_parser = subparsers.add_parser("search", help="Search ThePirateBay.")
    top_parser = subparsers.add_parser(
        "top", help="Browse ThePirateBay's top torrents."
    )

    search_parser.add_argument(
        "query", nargs="?", type=str, default=None, help="Search query."
    )
    top_parser.add_argument(
        "-r", "--recent", action="store_true", help="Top torrents in the last 48h."
    )
    top_parser.add_argument(
        "category",
        nargs="?",
        choices=[category.lower() for category in CATEGORIES_STRINGS],
        default=None,
        help="ThePirateBay category.",
    )
    return parser.parse_args(args)


def main():
    """Main Logic."""
    args = parse_args()

    if not CONFIG_DIR.is_dir():
        CONFIG_DIR.mkdir()
    if not CONFIG_FILE.is_file():
        write_default()

    try:
        tpb = TPB(url=args.url)
        if hasattr(args, "category"):
            # if top subparser
            if args.category is not None:
                category = args.category.upper()
            else:
                category = None
            torrent = tpb.top(category=category)
        elif hasattr(args, "query"):
            # if search subparser
            torrent = tpb.search(query=args.query)
        else:
            # if not subparser
            torrents = tpb.search_or_top()

        for torrent in torrents:
            tpb.action(torrent)
    except MenuError:
        pass
    except (URLError, HTTPError):
        print(
            f"Can't access '{args.url}', specify a different proxy with the -u flag.",
            file=sys.stderr,
        )



import shlex
from subprocess import Popen
from typing import List, Optional
from urllib.request import urlopen

from dynmen import Menu
from tpblite import CATEGORIES
from tpblite import TPB as TPBAPI
from tpblite.models.torrents import Torrent, Torrents

from .config import CONFIG
from .proxy import get_proxies
from .utils import torrent_format


class TPB:
    def __init__(self, url: Optional[str] = None):
        if url is None:
            if CONFIG["menu"].getboolean("use_tpb_proxy"):
                try:
                    url = get_proxies()[0]
                except Exception:
                    url = CONFIG["menu"]["tpb_url"]
            else:
                url = CONFIG["menu"]["tpb_url"]
        self.url = url
        if not self._check_url(self.url):
            raise ValueError(f"Cannot reach '{self.url}'.")
        self.tpb = TPBAPI(self.url)

    def get_menu(
        self,
        prompt: Optional[str] = None,
        lines: Optional[int] = None,
        multiple: bool = False,
        message: Optional[str] = None,
    ) -> Menu:
        """Create the dynamic menu object."""
        args = shlex.split(CONFIG["menu"]["command"])
        if "rofi" in args:
            if prompt is not None:
                args += ["-p", prompt]
            if lines is not None:
                args += ["-l", lines]
            if multiple:
                args += ["-multi-select"]
            if message is not None:
                args += ["-mesg", message]
        return Menu(args)

    @staticmethod
    def _check_url(url):
        return urlopen(url).getcode() == 200

    def search_or_top(self) -> str:  # pylint: disable=inconsistent-return-statements
        """Chose between top or search.

        Returns:
            Torrents matching either the search or the top category.
        """
        choices = {"Search": self.search, "Top": self.top}
        menu = self.get_menu(prompt="Select", lines=2)
        out = menu(choices)
        return out.value()

    def search(self, query: Optional[str] = None) -> Torrents:
        """Search for torrents.

        Args:
            query (optional): search query.

        Returns:
            The Torrents matching the search query.
        """
        if query is None:
            menu = self.get_menu(prompt="Search", lines=0)
            query = menu().selected()
        torrents = self.tpb.search(query)
        return self.select(torrents)

    def top(self, category: Optional[int] = None) -> Torrents:
        """Get the top torrents for a category.

        Args:
            category (optional): top category.

        Returns:
            The torrents for the selected categories.
        """
        if category is None:
            categories = [
                cat.strip() for cat in CONFIG["menu"]["categories"].split(",")
            ]
            if CONFIG["menu"].getboolean("categories_48h"):
                categories += [cat + " 48h" for cat in categories]
            categories = sorted(categories)
            menu = self.get_menu(prompt="Select", lines=len(categories))
            out = menu(categories)
            category = out.selected
        last_48 = "48h" in category
        category = category.split()[0]
        category = getattr(CATEGORIES, category)
        if not isinstance(category, int):
            category = category.ALL
        torrents = self.tpb.top(category=category, last_48=last_48)
        return self.select(torrents)

    def select(self, torrents: Torrents) -> List[Torrent]:
        """Select a torrent from a `Torrents` object.

        Args:
            torrents: `Torrents`from which to select a single torrent.

        Reuturns:
            Selected torrents.
        """
        torrents_formatted = {}
        for torrent in torrents:
            torrents_formatted[
                torrent_format(CONFIG["menu"]["torrent_format"], torrent)
            ] = torrent
        menu = self.get_menu(prompt="Select", multiple=True)
        out = menu(torrents_formatted)
        selected_out = []
        for selected in out.selected.split("\n"):
            selected_out.append(torrents_formatted[selected])
        return selected_out

    def action(self, torrent: Torrent) -> None:
        """Execute an action on `Torrent`.

        Args:
            torrent: `Torrent` instance on which to run the action.
        """
        actions = CONFIG["actions"]
        menu = self.get_menu(prompt="Select", lines=len(actions), message=torrent)
        out = menu(actions)
        command = torrent_format(out.value, torrent)
        Popen(command, shell=True)



import configparser

from .settings import CONFIG_FILE, DEFAULT_CONFIG


def write_default() -> None:
    """Write the default config to file."""
    parser = configparser.ConfigParser()
    parser.read_dict(DEFAULT_CONFIG)
    with CONFIG_FILE.open("w") as fp:
        parser.write(fp)


def load_config() -> configparser.ConfigParser:
    """Parse the config and return the ConfigParser instance.

    Returns:
        Parsed ConfigPraser instance.
    """
    parser = configparser.ConfigParser()
    parser.read_dict(DEFAULT_CONFIG)
    parser.read(CONFIG_FILE)
    return parser


CONFIG = load_config()



from typing import List
from urllib.request import Request, urlopen

from lxml.etree import HTML

from .settings import HEADERS, PROXY_URL


def get_proxies() -> List[str]:
    req = Request(PROXY_URL, None, HEADERS)
    return HTML(urlopen(req).read()).xpath("//td[@class='site']/a/@href")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rofi_tpb']

package_data = \
{'': ['*']}

install_requires = \
['dynmen>=0.1.5,<0.2.0',
 'lxml>=4.6.1,<5.0.0',
 'tpblite>=0.6,<0.7',
 'traitlets>=5.0.4,<6.0.0']

entry_points = \
{'console_scripts': ['rofi-tpb = rofi_tpb.cli:main']}

setup_kwargs = {
    'name': 'rofi-tpb',
    'version': '0.1.1',
    'description': 'Dynamic menu interface for TPB, built with rofi in mind.',
    'long_description': "# rofi-tpb\n\nDynamic menu interface for The Pirate Bay, built with [`rofi`](https://github.com/davatorium/rofi) in mind.\n\n# Install\n\n```shell\npip install rofi-tpb\n```\n\nConsider using [`pipx`](https://github.com/pipxproject/pipx):\n```shell\npipx install rofi-tpb\n```\n\n# Usage\n\nPrompt to either search tpb or browse tpb's top torrents:\n```shell\nrofi-tpb\n```\n\nPrompt for search query:\n```shell\nrofi-tpb search\n```\n\nSearch for ubuntu related torrents:\n```shell\nrofi-tpb search ubuntu\n```\n\nBrowse tpb's top torrents across all categories:\n```shell\nrofi-tpb top all\n```\n\nBrowse tpb's recent top torrents across all categories in the last 48h:\n```shell\nrofi-tpb top all -r\n```\n\nCheck the help for details:\n```shell\nrofi-tpb --help\n...\nrofi-tpb search --help\n...\nrofi-tpb top --help\n...\n```\n\n# Dependencies\n\n* `python >= 3.7`\n* `tpblite`\n* `dynmen`\n* `lxml`\n* `traitlets` (undeclared dependency of `dynmen`)\n\n\n# Configuration\n\n`rofi-tpb` stores its config in `$XDG_CONFIG_HOME/rofi-tpb/config.ini`.\n\nThe default configuration is the following:\n\n```ini\n[menu]\ncommand = rofi -dmenu -i\ntorrent_format = {title:<70} 📁{filesize:<10} 🔽{seeds:<4} 🔼{leeches:<4}\nuse_tpb_proxy = True\ntpb_url = https://thepiratebay0.org\ncategories = All, APPLICATIONS, AUDIO, GAMES, OTHER, PORN, VIDEO\ncategories_48h = True\n\n[actions]\nadd = xdg-open '{magnetlink}'\nopen = xdg-open '{url}'\n```\n\n * `menu.command`: the dynamic menu command which should read from `stdin` and output to `stdout`, if you want to use `dmenu` instead of `rofi` then adjust this command accordingly.\n * `menu.torrent_format`: text representation of a torrent in the dynamic menu, **accepts torrent string format keys.**\n * `menu.use_tpb_proxy`: if True will use the first tpb url as found on https://piratebayproxy.info/.\n * `menu.tpb_url`: tpb url, if `use_tpb_proxy` is True acts as a fallback url in case https://piratebayproxy.info/ is unavailable or the parsing fails.\n * `menu.categories`: comma separated list of tpb categories, when browsing the top torrents.\n * `menu.categories_48h`: add last 48h top torrent categories.\n * `actions.*`: commands to run on the selected torrent, **accepts torrent string format keys.**\n\nAvailable torrent string format keys:\n * `{title}`: torrent title\n * `{filesize}`: torrent file size\n * `{seeds}`: number of seeders\n * `{leeches}`: number of leechers\n * `{uploader}`: torrent uploader\n * `{upload_date}`: upload date\n * `{url}`: torrent's tpb url\n * `{magnetlink}`: torrent magnet link\n\nIf the `menu.command` uses `rofi`, `rofi-tpb` will use `rofi`'s `-multiple-select` flag to allow for selecting multiple torrents.\n",
    'author': 'Loic Coyle',
    'author_email': 'loic.coyle@hotmail.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/loiccoyle/rofi-tpb',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os
from pathlib import Path
from random import choice

CONFIG_DIR = Path(os.getenv("XDG_CONFIG_HOME"), Path.home() / ".config") / "rofi-tpb"
CONFIG_FILE = CONFIG_DIR / "config.ini"

DEFAULT_CONFIG = {
    "menu": {
        "command": "rofi -dmenu -i",
        "torrent_format": "{title:<70} 📁{filesize:<10} 🔽{seeds:<4} 🔼{leeches:<4}",
        "use_tpb_proxy": True,
        "tpb_url": "https://thepiratebay0.org",
        "categories": "All, APPLICATIONS, AUDIO, GAMES, OTHER, PORN, VIDEO",
        "categories_48h": True,
    },
    "actions": {"add": "xdg-open '{magnetlink}'", "open": "xgd-open '{url}'"},
}

PROXY_URL = "https://piratebayproxy.info/"

USER_AGENTS = (
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
    "AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/60.0.3112.113 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
    "AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/60.0.3112.101 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64) "
    "AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/60.0.3112.113 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) "
    "AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/60.0.3112.113 Safari/537.36",
)

HEADERS = {"User-Agent": choice(USER_AGENTS)}

CATEGORIES_STRINGS = [
    "ALL",
    "APPLICATIONS",
    "AUDIO",
    "GAMES",
    "OTHER",
    "PORN",
    "VIDEO",
]



from tpblite.models.torrents import Torrent


def torrent_format(string: str, torrent: Torrent) -> str:
    """Format a torrent into a string.

    Args:
        string: string describing the format.
            Possible format keys:
                title: torrent.title
                filesize: torrent.filesize
                seeds: torrent.leeches
                leeches: torrent.leeches
                uploader: torrent.uploader
                upload_date: torrent.upload_date
                url: torrent.url
                magnetlink: torrent.magnetlink

    Returns:
        formatted string.
    """
    return string.format(
        title=torrent.title.strip(),
        filesize=torrent.filesize,
        seeds=torrent.seeds,
        leeches=torrent.leeches,
        uploader=torrent.uploader,
        upload_date=torrent.upload_date,
        url=torrent.url,
        magnetlink=torrent.magnetlink,
    )
